#! /usr/bin/env python3

from argparse import ArgumentParser
from pathlib import Path
from typing import List, Union

from emod_api.channelreports.utils import property_report_to_csv
from emod_api.channelreports.channels import ChannelReport

def main(source_file: Union[Path, str], csv_file: Union[Path, str], channels: List[str]=None, rollup: List[str]=None, aggregated: bool=True, normalize: bool=True, transpose: bool=False) -> None:

    if aggregated:
        # Sum all sub-channels (different IP values) of each channel, optionally output "Statistical Population" as "normalize" for normalization.
        property_report_to_csv(source_file, channels, rollup, csv_file, normalize, transpose)
    else:
        # Write _all_ [sub-]channels as a row in the resulting CSV file.
        report = ChannelReport(str(source_file))
        report.to_csv(csv_file, channels=None, transpose=transpose)

    return

if __name__ == "__main__":

    parser = ArgumentParser()
    parser.add_argument("filename", nargs="?", default=Path("PropertyReport.json"), type=Path, help="property report filename [PropertyReport.json]")
    parser.add_argument("-o", "--output", default=Path("PropertyReport.csv"), type=Path, help="output filename [PropertyReport.csv]")
    parser.add_argument("-c", "--channel", action="append", help="channel(s) to display [Infected] (ignored with 'raw' option)", metavar="channelName", dest="channels")
    parser.add_argument("-r", "--rollup", action='append', help="IP(s) under which to roll up other IP keys and values")
    parser.add_argument("--raw", action="store_false", dest="aggregated", help="do _not_ aggregate property report sub-channels")
    parser.add_argument("-n", "--no-normalization", action="store_false", dest="normalize", help="do _not_ output 'normalize' channel for normalization by statistical population")
    parser.add_argument("-t", "--transpose", action="store_true", help="output channels as columns rather than rows")

    args = parser.parse_args()

    if not args.channels:
        args.channels = ['Infected']

    if args.rollup is not None and len(args.rollup) == 1 and args.rollup[0].lower() == "all":
        args.rollup = []

    main(args.filename, args.output, args.channels, args.rollup, args.aggregated, args.normalize, args.transpose)
